/*
 *  $Id: cart_shift.c,v 1.4 1993/11/12 12:35:46 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Cart_shift - given a shift direction and amount, returns the
                 shifted source and destination ranks

@*/
int MPI_Cart_shift ( comm, direction, displ, source, dest )
MPI_Comm  comm;
int               direction;
int               displ;
int              *source;
int              *dest;
{
  int i, rank, size;
  int source_position, dest_position, save_position, periodic;

  /* Check for valid arguments */
  if ( (comm                      == (MPI_Comm)0)             ||
       (comm->topology.type       != MPI_CART)                        ||
       (direction                 <  0)                               ||
       (direction                 >= (comm->topology.tag.cart.ndims)) ||
       (dest                      == (int *)0)                        ||
       (source                    == (int *)0) )
    return (MPI_SUCCESS);

  /* Check the case for a 0 displacement */
  MPI_Comm_rank (comm, &rank);
  if (displ == 0) {
    (*source) = (*dest) = rank;
    return (MPI_SUCCESS);
  }

  /* Get ready for shifting */
  MPI_Comm_size (comm, &size);
  periodic = comm->topology.tag.cart.periods[direction];
  save_position = source_position = dest_position = 
    comm->topology.tag.cart.position[direction];
  
  /* Shift for the destination */
  dest_position += displ;
  if ( dest_position >= comm->topology.tag.cart.dims[direction] ) {
    if ( periodic )
      dest_position %= comm->topology.tag.cart.dims[direction];
    else
      dest_position = MPI_PROCNULL;
  }
  else if ( dest_position < 0 ) {
    if ( periodic )
      dest_position += comm->topology.tag.cart.dims[direction];
    else
      dest_position = MPI_PROCNULL;
  }
  comm->topology.tag.cart.position[direction] = dest_position;
  MPI_Cart_rank ( comm, comm->topology.tag.cart.position, dest );

  /* Shift for the source */
  source_position -= displ;
  if ( source_position >= comm->topology.tag.cart.dims[direction] ) {
    if ( periodic )
      source_position %= comm->topology.tag.cart.dims[direction];
    else
      source_position = MPI_PROCNULL;
  }
  else if ( source_position < 0 ) {
    if ( periodic )
      source_position += comm->topology.tag.cart.dims[direction];
    else
      source_position = MPI_PROCNULL;
  }
  comm->topology.tag.cart.position[direction] = source_position;
  MPI_Cart_rank ( comm, comm->topology.tag.cart.position, source );

  /* Restore my position */
  comm->topology.tag.cart.position[direction] = save_position;

  return (MPI_SUCCESS);
}


